function animate_cases(animationType, case1, case2, nAnimationSteps, legendStr)
%ANIMATE_CASES Figure animation of two virmlab cases.
%
%   function animate_cases(animationType, case1, case2, nAnimationSteps, ...
%       legendStr)
%
%   Animate cases.

% By Ulf Carlberg.

if nargin<5, legendStr = {}; end

assert(all(case1.nSamplesVector==case2.nSamplesVector), ...
    'Both cases must use the same nSamplesVector.');

% Define figure and pause function.
hf = figure('OuterPosition', figpos('big'));
set(hf, 'Toolbar', 'figure');
pause_figure_animation(hf, 'on');

% Animation loop.
for i = round(linspace(find(case1.nSamplesVector>100, 1, 'first'), ...
        numel(case1.nSamplesVector), nAnimationSteps))
    nSamples = case1.nSamplesVector(i);

    subplot(2, 2, 1);
    plot_cdf(case1.v(1:nSamples,:,:));
    grid on;
    axis([-40 10 1e-3 1e0]);
    bigText(0.01, 0.98, [inputname(2) ', N = ' num2str(nSamples)]);

    subplot(2, 2, 2);
    plot_cdf(case2.v(1:nSamples,:,:));
    grid on;
    axis([-40 10 1e-3 1e0]);
    bigText(0.01, 0.98, [inputname(3) ', N = ' num2str(nSamples)]);

    % Show legend after animation is finished, because otherwise the legend
    % will slow down the animation speed significantly.
    if i==case1.nSamplesVector(end)
        s = legendStr;
    else
        s = {};
    end

    subplot(2, 2, 3);
    plot_parameter(animationType, case1, i, s);
    bigText(0.01, 0.98, inputname(2));

    subplot(2, 2, 4);
    plot_parameter(animationType, case2, i, s);
    bigText(0.01, 0.98, inputname(3));

    drawnow;
end
pause_figure_animation(hf, 'off');

% Define callback commands for the Control Buttons.
st = dbstack; % to get name of the script that called this function
if length(st)<2 % called from command line
    editStr = 'commandwindow;';
else
    editStr = ['edit(''' st(end).file ''');'];
end
infoStr = ['fprintf(''' virmlab() '''); commandwindow;'];
figStr = sprintf('figure(''OuterPosition'', [%d %d %d %d]); ', ...
    figpos('middle'));
plotWavesStr = sprintf([figStr ...
    'subplot(1, 2, 1); plot_environment(%s); ' ...
    'subplot(1, 2, 2); plot_environment(%s); '], inputname(2), inputname(3));
plotPosStr = sprintf([figStr ...
    'subplot(1, 2, 1); plot_positions(%s.p); ' ...
    'subplot(1, 2, 2); plot_positions(%s.p); '], inputname(2), inputname(3));
plotAntStr = sprintf([figStr ...
    'subplot(1, 2, 1); plot_farfield_inpos(%s); ' ...
    'subplot(1, 2, 2); plot_farfield_inpos(%s); '], ...
    inputname(2), inputname(3));

% Define the Control Buttons in the figure window.
bt([10 185], plotWavesStr, 'Environment', 'Show Waves/Clusters in Environment');
bt([10 150], plotPosStr, 'Positions', 'Show Positions and Orientations');
bt([10 115], plotAntStr, 'Antennas', 'Show Antenna Radiation Patterns');
bt([10 80], editStr, 'Edit Input', 'Edit Input Script');
bt([10 45], infoStr, 'Info', 'Information (Matlab Command Window)');
bt([10 10], 'close all;', 'Close All', 'Close all Figures');



%-------------------------------------------------------------------------------
function bigText(x, y, s)
text(x, y, s, 'BackgroundColor', [.6 .6 1.], 'FontSize', 14, ...
    'VerticalAlignment', 'top', 'Units', 'normalized');
%-------------------------------------------------------------------------------
function bt(pos, call, name, tip)
uicontrol('Style', 'pushbutton', 'String', name, 'TooltipString', tip, ...
    'Position', [pos(1) pos(2) 70 25], 'Callback', call);
%-------------------------------------------------------------------------------
