function c = calc(cin, nSamplesVector, ant1, ant2, interp_method)
%CALC Calculate received voltages on antenna ports and other parameters.
%
%   function c = calc(cin, nSamplesVector, ant1, ant2)
%
%   Calculate Voltages at ports, Mean Effective Directivity, and Capacity of
%   case "cin". Calculate Correlation, Efficiency Ratio, and Apparent Diversity
%   Gain between antennas "ant1" and "ant2" of case "cin". "nSamplesVector" is
%   a list of the number of samples that the parameters should be calculated
%   for.

% By Ulf Carlberg.

if nargin<5, interp_method = []; end

% Copy structure and add the number of samples vector.
c = cin;
c.nSamplesVector = nSamplesVector;

% Calculate Voltages at ports.
if isfield(c, 'w')
    v = calc_v(c, interp_method);
else
    v = calc_v_clusters(c);
end

% TODO: Generalize.
if size(v, 1)==1
    siz = size(v);
    v = reshape(v, [siz(2) 1 siz(3:end)]);
end
c.v = v;

% Calculate capacity for all voltages.
if size(c.v, 3)==1
    v2 = shiftdim(c.v, -1);
else
    v2 = shiftdim(c.v, 2);
end
siz = size(v2);
cap = capacity(reshape(v2, [siz(1) 1 siz(2:end)]));

for i = 1:numel(nSamplesVector)
    nSamples = nSamplesVector(i);

    % Calculate Mean Effective Directivity.
    c.med(i, :) = mean(abs(c.v(1:nSamples, :)).^2, 1);

    % Calculate Capacity.
    c.capacity(i, :) = mean(cap(1:nSamples, :), 1);

    for j = 1:size(c.v, 2)
        
        % Calculate Correlation.
        c.correlation(i, j, :, :) = corrcoef(squeeze(c.v(1:nSamples, j, :)));

        % Calculate Efficiency Ratio.
        tmp = sort(c.med(i, [ant1 ant2]), 2);
        c.eRatio(i, 1) = tmp(1) ./ tmp(2);

        % Calculate Apparent Diversity Gain (three alternative formulas).
        c.adg1(i, j) = 10.48*sqrt(1 - abs(c.correlation(i, j, ant1, ant2)).^2);
        c.adg2(i, j) = ...
            10.48 * sqrt(1 - 0.99*abs(c.correlation(i, j, ant1, ant2)).^2);
        c.adg3(i, j) = sqrt(1+(10.48^2-1).*c.eRatio(i).* ...
            (1-abs(c.correlation(i, ant1, ant2)).^2));
    end
end
