function [v_lin, v_rhc, v_lhc] = calc_rlos(a, n, ifreq)
%CALC_RLOS Calculate received voltage in random line-of-sight (LOS).
%
%   function [v_lin, v_rhc, v_lhc] = calc_rlos(a, n, ifreq)
%
%   Calculate received voltage for antenna "a" from random line-of-sight waves
%   by using regular sampling of the far-field sphere. "n" is the number of
%   samples of the sphere (default 1000) and "ifreq" is the frequency number in
%   antenna structure "a" used for the calculation (default 1). Output is the
%   received voltages for linear, right hand circular, and left hand circular,
%   respectively.

% By Ulf Carlberg.

% Default arguments.
if nargin<2 || isempty(n), n = 1000; end
if nargin<3 || isempty(ifreq), ifreq = 1; end

% Regular sampling of sphere.
[theta, phi] = sph_regsamp(n);
theta = theta*pi/180;  %%%deg2rad%%%
phi = phi*pi/180;  %%%deg2rad%%%

% Allow phi interval [-pi..pi] or [0..2*pi] of antenna.
if any(a{1}.phi>pi+0.01)  %%%deg2rad%%%
    phi(phi<0) = phi(phi<0) + 2*pi;  %%%deg2rad%%%
end

% Define psi angle (the polarization angle for linear polarization). We want
% different values for the different incoming directions, and we want it to vary
% "irregular". Therefore, the golden ratio is used here.
dpsi = 90*(sqrt(5)-1)/2;
psi = dpsi*(1:n)';

v_lin = zeros(1, n, numel(a));
v_rhc = zeros(1, n, numel(a));
v_lhc = zeros(1, n, numel(a));
for iant = 1:numel(a)
    % Evaluate far-field function with the regular sampling.
    Eth = interp2(a{iant}.phi, a{iant}.theta, a{iant}.Eth(:,:,ifreq), ...
        phi, theta, 'spline');
    Eph = interp2(a{iant}.phi, a{iant}.theta, a{iant}.Eph(:,:,ifreq), ...
        phi, theta, 'spline');

    % Calculate the random LOS from linear/RHC/LHC polarization.
    v_lin(1,:,iant) = Eth.*cosd(psi) + Eph.*sind(psi);
    v_rhc(1,:,iant) = (Eth - 1j*Eph) / sqrt(2);
    v_lhc(1,:,iant) = (Eth + 1j*Eph) / sqrt(2);
end

end
