function v = calc_v(c, interp_method)
%CALC_V Calculate received voltages on antenna ports for a virmlab case.
%
%   function v = calc_v(c, interpMethod)
%
%   Calculate the port voltages for case "c". "c" defines positions, antennas,
%   and incoming plane waves.
%
%   The interpolation method may be chosen by defining "interpMethod". Default
%   is '*nearest'. See the help section of interp2 for more information.
%
%   The output matrix "v" have the following dimensions:
%       dim 1: Realizations (rows)
%       dim 2: Positions (columns)
%       dim 3: Ports
%       dim 4: Frequencies

% By Ulf Carlberg.

if nargin<2 || isempty(interp_method)
    interp_method = '*nearest';
end

nFreq = numel(c.a(1).f);

% Initialize output variables.
v = complex(zeros([c.w.nRealizations size(c.p.x) nFreq]));

% Unit vectors for direction of incoming plane waves.
[uRx, uRy, uRz] = unit_vector_rhat(c.w.theta, c.w.phi);

for iAnt = 1:size(c.p.x, 2)

    % If "phi" is in the interval [0, 2*pi] (instead of [-pi, pi]) for this
    % antenna, we must also change phi for the incoming waves to this interval
    % to make the interpolation work.
    w_phi = c.w.phi;
    w_theta = c.w.theta;
    a_phi_max = max(c.a(iAnt).phi(:));
    a_phi_min = min(c.a(iAnt).phi(:));
    w_phi(w_phi>a_phi_max) = w_phi(w_phi>a_phi_max) - 2*pi;
    w_phi(w_phi<a_phi_min) = w_phi(w_phi<a_phi_min) + 2*pi;

    % The rotation of the far-field in the following loop can be time-consuming,
    % so we should try to avoid it.
    oldAlpha = NaN;
    oldTheta = NaN;
    oldPhi = NaN;
    [~, ipOrder] = sortrows(...
        [c.p.alpha(:,iAnt) c.p.theta(:,iAnt) c.p.phi(:,iAnt)]);

    for ip = ipOrder'

        % Rotate antenna far-field function according to current position, if
        % needed.
        if oldAlpha~=c.p.alpha(ip, iAnt) || oldTheta~=c.p.theta(ip, iAnt) || ...
                oldPhi~=c.p.phi(ip, iAnt)
            a = rotate_farfield(c.a(iAnt), ...
                c.p.alpha(ip, iAnt), c.p.theta(ip, iAnt), c.p.phi(ip, iAnt));
            oldAlpha = c.p.alpha(ip, iAnt);
            oldTheta = c.p.theta(ip, iAnt);
            oldPhi = c.p.phi(ip, iAnt);
        end

        % Account for phase-change of incoming plane waves at current position.
        phaseChange = exp(2j*pi*( ...
            uRx*c.p.x(ip,iAnt) + uRy*c.p.y(ip,iAnt) + uRz*c.p.z(ip,iAnt) ));

       for iFreq = 1:nFreq
            % Evaluate far-field function for each plane wave and calculate the
            % scalar product.
            E = interp2_safe(a.phi, a.theta, a.Eth(:,:,iFreq), ...
                w_phi, w_theta, interp_method) .* c.w.E_theta + ...
                interp2_safe(a.phi, a.theta, a.Eph(:,:,iFreq), ...
                w_phi, w_theta, interp_method) .* c.w.E_phi;

            % Get final voltages in ports.
            v(:, ip, iAnt, iFreq) = sum(E.*phaseChange, 1);
       end
    end
end
