function v = calc_v_clusters(c, interp_method)
%CALC_V_CLUSTERS Calculate the received voltages in a cluster environment.
%
%   function v = calc_v_clusters(c, interp_method)
%
%   Calculate the port voltages for case "c". "c" defines positions, antennas,
%   and clusters.
%
%   The interpolation method may be chosen by defining "interpMethod". Default
%   is '*nearest'. See the help section of interp2 for more information.

% By Ulf Carlberg.

if nargin<2 || isempty(interp_method)
    interp_method = '*nearest';
end

% Initialize output variables.
v = complex(zeros([1 size(c.p.x)]));

for iAnt = 1:size(c.p.x, 2)
    
    % The rotation of the far-field in the following loop can be time-consuming, so we should try to avoid it.
    oldAlpha = NaN;
    oldTheta = NaN;
    oldPhi = NaN;
    [~, ipOrder] = sortrows([c.p.alpha(:,iAnt) c.p.theta(:,iAnt) c.p.phi(:,iAnt)]);

    for ip = ipOrder'
    
        % Rotate antenna far-field function according to current position, if needed.
        if oldAlpha~=c.p.alpha(ip, iAnt) || oldTheta~=c.p.theta(ip, iAnt) || oldPhi~=c.p.phi(ip, iAnt)
            a = rotate_farfield(c.a(iAnt), c.p.alpha(ip, iAnt), c.p.theta(ip, iAnt), c.p.phi(ip, iAnt));
            oldAlpha = c.p.alpha(ip, iAnt);
            oldTheta = c.p.theta(ip, iAnt);
            oldPhi = c.p.phi(ip, iAnt);
        end
    
        % Account for amplitude and phase-change from cluster points to current position and calculate from which angles
        % the waves from the cluster points arrive at the current position.
        [c_r, c_theta, c_phi] = xyz2rthetaphi(c.c.x-c.p.x(ip,iAnt), c.c.y-c.p.y(ip,iAnt), c.c.z-c.p.z(ip,iAnt));
        phaseChange = exp(2j*pi*c_r);

        % If "phi" is in the interval [0, 2*pi] (instead of [-pi, pi]) for this antenna, we must also change phi for the 
        % incoming waves to this interval to make the interpolation work.
        a_phi_max = max(c.a(iAnt).phi(:));
        a_phi_min = min(c.a(iAnt).phi(:));
        c_phi(c_phi>a_phi_max) = c_phi(c_phi>a_phi_max) - 2*pi;
        c_phi(c_phi<a_phi_min) = c_phi(c_phi<a_phi_min) + 2*pi;

        % Evaluate far-field function for a plane wave from each cluster point.
        E = interp2_safe(a.phi, a.theta, a.Eth, ...
            c_phi, c_theta, interp_method) .* c.c.E_theta + ...
            interp2_safe(a.phi, a.theta, a.Eph, ...
            c_phi, c_theta, interp_method) .* c.c.E_phi;

        % Get final voltages in ports.
        v(:, ip, iAnt) = sum(E.*phaseChange, 1);
    end
end
