function C = capacity(H, SNR)
%CAPACITY Calculate capacity of channel matrix for a given SNR (signal-to-noise ratio).
%
%   function C = capacity(H, SNR)
%
%   Calculates the capacity [bit/s/Hz] of MIMO channel matrix "H" for the
%   signal-to-noise ratio "SNR". (It is assumed uniform power allocation on the
%   transmitter side.) The channel matrix "H":s rows correspond to the receive
%   ports, and the columns of the channel matrix "H" correspond to the transmit
%   ports. Additional dimensions of "H" corresponds to different cases and will
%   give additional dimensions of the output variable "C".
%
%   The "SNR" ratio shall be given for the linear power ratio. If it is not
%   specified, it is assumed to be 31.6228 (=15 dB). "SNR" can be a
%   scalar, a square matrix with dimension [nRx nRx] or a 3-dimensional matrix
%   with size [nRx nRx nSNRcases] where nRx = size(H, 1).
%
%   The output matrix "C" will have dimension [sizeH(3:end) nSNRcases]
%   where sizeH = size(H).
%
%   References:
%   [1] "On the Capacity of the MIMO Channel - A Tutorial Introduction",
%       by Bengt Holter, Norwegian University of Science and Technology,
%       Department of Telecommunications
%   [2] Equation (10.13) in "Wireless Communications" by Andrea Goldsmith,
%       Cambridge University Press 2005.

% By Ulf Carlberg.

% Default SNR is 15 dB.
if nargin<2, SNR = 10^(15/10); end

% Get number of receive and transmit ports.
nRx = size(H, 1);
nTx = size(H, 2);

% Check SNR input dimension.
assert(size(SNR, 1)==size(SNR, 2) && (size(SNR, 1)==1 || ...
    size(SNR, 1)==nRx) && ndims(SNR)<=3, 'Wrong dimension of "SNR".');

% Init output.
siz = size(H);
siz = siz(3:end);
C = zeros(prod(siz), size(SNR, 3));

% Calculate MIMO capacity with a formula from [1] and [2]:
for iH = 1:prod(siz)
    HH_over_nTx = (H(:, :, iH) * H(:, :, iH)') / nTx;
    for iSNR = 1:size(SNR, 3)
        % Take the real part below, because numerical inaccuracies might give
        % a small imaginary residual.
        C(iH, iSNR) = log2(real(det( ...
            eye(nRx) + SNR(:, :, iSNR) .* HH_over_nTx )));
    end
end

% Reshape output.
C = reshape(C, [siz size(SNR, 3) 1]);
