function corr = farfield_correlation(a1, a2)
%FARFIELD_CORRELATION Calculate the correlation of two antenna far-fields in a rich isotropic multi-path environment.
%
%   function corr = farfield_correlation(a1, a2)
%
%   Calculate far-field correlation [1] between antenna structure "a1" and
%   antenna structure "a2" in rich isotropic multi-path.
%
%   [1] Radiation efficiency, correlation, diversity gain and capacity of a 
%       six-monopole antenna array for a MIMO system: theory, simulation and 
%       measurement in reverberation chamber, K. Rosengren and P.-S. Kildal,
%       IEE Proc. Microw. Antennas Propag. Vol. 152, No. 1, February 2005.

% By Ulf Carlberg.

% Resample a2 the same as a1.
nTheta = size(a1.theta, 1);
nPhi   = size(a1.phi  , 1);
a2 = rotate_farfield(a2, 0, 0, 0, nTheta, nPhi);

% Calculate the integrals. We can skip the weigths because they will cancel each
% other in the division. (From Equation (14) in [1].)
siz = size(a1.Eth); siz(1) = 1;
tmp = repmat(sin(a1.theta), siz);
int12 = trapz(trapz((a1.Eth.*conj(a2.Eth) + a1.Eph.*conj(a2.Eph)) .* tmp, 1), 2);
int11 = trapz(trapz((a1.Eth.*conj(a1.Eth) + a1.Eph.*conj(a1.Eph)) .* tmp, 1), 2);
int22 = trapz(trapz((a2.Eth.*conj(a2.Eth) + a2.Eph.*conj(a2.Eph)) .* tmp, 1), 2);

% Calculate from Equation (14) in [1]. Take the real part because there might be 
% a small imaginary residual due to limited accuracy of floating point numbers.
corr = real( int12 / sqrt(int11*int22) );
