function Gi = fft_interp_periodic_function(G, N)
%FFT_INTERP_PERIODIC_FUNCTION Interpolates periodic function using FFT (BOR-expansion of antenna far-fields).
%
%   function Gi = fft_interp_periodic_function(G, N)
%
%   Interpolate the periodic function "G" using Fourier transform along the
%   first dimension to "N" samples. The function "G" is assumed to be sampled
%   at evenly spaced intervals.
%
%   This function can be used to do a BOR-expansion of an antenna far-field
%   function over phi-angles. (BOR = Bodies of Revolution, see [2])
%
%   References:
%   [1] Appendix A4 in "Spherical Near-Field Antenna Measurements" edited by
%       J.E. Hansen, (IEE electromagnetic waves series; 26),
%       1988 Peter Peregrinus Ltd., London, United Kingdom.
%   [2] "Foundations of Antennas - A Unified Approach" by Per-Simon Kildal,
%       2000 Studentlitteratur.
%
%   See also IFFT, FFT.

% By Ulf Carlberg.

% Get coefficients by inverse Fourier transform.
c = ifft(G, [], 1);

% Pad coefficients with zeros in the middle if resolution should be increased,
% or take away middle elements if resolution should be decreased. An odd number
% of samples is a symmetric case, because "c(1, :)" is the center element(s).
% For even number of samples we have to split or merge two samples.
sz = size(G);
if sz(1)<N % increase resolution
    N1 = ceil(sz(1)/2);
    N2 = floor(sz(1)/2) + 2;
    Mz = prod(sz(2:end));
    if isodd(sz(1)) % odd number of input samples
        c = [c(1:N1, :) ; zeros(N-sz(1), Mz) ; c(N2:end, :)];
    else % even number of input samples
        c = [c(1:N1, :) ; c(N1+1, :)/2 ; zeros(N-sz(1)-1, Mz) ; ...
            c(N1+1, :)/2 ; c(N2:end, :)];
    end
elseif sz(1)>N % decrease resolution
    N1 = ceil(N/2);
    N2 = sz(1) - ceil(N/2) + 2;
    if isodd(N) % odd number of output samples
        c = [c(1:N1, :) ; c(N2:end, :)];
    else % even number of output samples
        c = [c(1:N1, :) ; (c(N1+1, :) + c(N2-1, :))/2 ; c(N2:end, :)];
    end
end

% Fourier transform back (and reshape).
Gi = reshape(fft(c, [], 1), [N sz(2:end)]);
