function a = gen_farfield(antType, nTheta, nPhi)
%GEN_FARFIELD Generate far-field of (incremental) dipoles, isotropic antennas, Huygens sources, cosn.
%
%   function a = gen_farfield(antType, nTheta, nPhi)
%
%   Generate antenna far-field of antenna type "antType". Return a
%   structure. "nTheta" and "nPhi" are optional arguments specifying the
%   number of samples for the theta- and phi-angles.
%
%   "antType"                             Desription
%   ---------------------------------------------------------------------------
%   'isotropic theta'                   - isotropic antenna, theta-polarized
%   'isotropic phi'                     - isotropic antenna, phi-polarized
%   'incremental electric monopole'     - z-directed, radiating for z>=0
%   'incremental electric dipole'       - z-directed
%   'incremental magnetic dipole'       - z-directed
%   'incremental magnetic dipole on gp' - z-directed
%   'incremental Huygens source'        - radiating in z-direction, x-polarized
%   'electric dipole'                   - z-directed
%   'magnetic dipole'                   - z-directed
%   'cosn X'                            - Pencil beam of "cos(theta/2)^n"-shape,
%                                         x-polarized, with "X" half beam width
%                                         in degrees (at -3 dB), e.g., 'cosn 40'
%                                         gives a pencil beam with 40 degrees
%                                         half beam width at the -3 dB level
%                                         [2].
%
% References:
% [1] Foundations of Antennas - A Unified Approach, Per-Simon Kildal,
%     Studentlitteratur 2000.

% By Ulf Carlberg.

% Define optional arguments.
if nargin<2 || isempty(nTheta), nTheta = 20;       end
if nargin<3 || isempty(nPhi),   nPhi   = 2*nTheta; end

% Save antenna type in structure.
a.type = 'farfield';
a.name = antType;
a.info.generated = 'Far-field generated with virmlab.';
a.f = NaN;

% Make a grid of theta and phi angles.
a.theta = linspace(0, pi, nTheta)';
a.phi = linspace(-pi, pi, nPhi)';
[theta2D, phi2D] = ndgrid(a.theta, a.phi);

% Define radiation pattern.
if strcmpi(antType, 'isotropic theta')
    Eth = ones(nTheta, nPhi);
    Eph = zeros(nTheta, nPhi);
elseif strcmpi(antType, 'isotropic phi')
    Eth = zeros(nTheta, nPhi);
    Eph = ones(nTheta, nPhi);
elseif strcmpi(antType, 'incremental electric monopole')
    Eth = (theta2D<=pi/2) .* sqrt(3) .* sin(theta2D);
    Eph = zeros(nTheta, nPhi);
    if isodd(nTheta)
        warning('virmlab:gen_farfield', ...
            'Odd nTheta makes the power integral inaccurate.');
    end
elseif strcmpi(antType, 'incremental electric dipole')
    Eth = sqrt(1.5) * sin(theta2D);
    Eph = zeros(nTheta, nPhi);
elseif strcmpi(antType, 'incremental magnetic dipole')
    Eth = zeros(nTheta, nPhi);
    Eph = sqrt(1.5) * sin(theta2D);
elseif strcmpi(antType, 'incremental magnetic dipole on gp')
    Eth = zeros(nTheta, nPhi);
    Eph = sqrt(3) * sin(theta2D);
    Eph(phi2D<0) = 0;
elseif strcmpi(antType, 'incremental Huygens source')  % (='cosn' for n=2)
    ExPrim = sqrt(3) * cos(theta2D/2).^2;
    Eth =  cos(phi2D).*ExPrim;
    Eph = -sin(phi2D).*ExPrim;
elseif strcmpi(antType, 'electric dipole')
    Eth = 1.28098527 * cos(0.5*pi*cos(theta2D)) ./ sin(theta2D);
    Eth(  1, :) = 0;
    Eth(end, :) = 0;
    Eph = zeros(nTheta, nPhi);
elseif strcmpi(antType, 'magnetic dipole')
    Eth = zeros(nTheta, nPhi);
    Eph = 1.28098527 * cos(0.5*pi*cos(theta2D)) ./ sin(theta2D);
    Eph(  1, :) = 0;
    Eph(end, :) = 0;
elseif strncmpi(antType, 'cosn ', 5) % cos(theta/2)^n
    halfBeamWidth = str2double(antType(6:end))*pi/180;
    assert(5*pi/180<=halfBeamWidth && halfBeamWidth<=pi, ...
        'virmlab:gen_farfield', ...
        'The parameter "halfBeamWidth" is outside the allowed range.');
    n = log(sqrt(1/2)) / log(cos(halfBeamWidth/2));
    ExPrim = sqrt(n+1) * cos(theta2D/2).^n;
    Eth =  cos(phi2D).*ExPrim;
    Eph = -sin(phi2D).*ExPrim;
else
    error('Unknown antenna type.');
end
a.Eth = complex(Eth);
a.Eph = complex(Eph);

% Define the multiplicative directivity factor (in linear scale).
a.D0 = 1;

% Integrate the far-field to get the radiated power.
a.power.radiated = integrate_farfield(a);
