function p = gen_positions(x, y, z, alpha, theta, phi)
%GEN_POSITIONS Generate positions (spatial translation and rotation) by expansion of input matrices.
%
%   function p = gen_positions(x, y, z, alpha, theta, phi)
%
%   Generates positions and direction angles in the structure "p". The input
%   matrices is automatically expanded so that all have the same size.
%   * Dimension 1 (rows) corresponds to when the DUT is moving around.
%   * Dimension 2 (columns) corresponds to the position of the different 
%     antennas for the DUT.
%
%   Examples:
%
%   * A single-port antenna in a fixed position (origin), no rotation:
%     "gen_positions(0, 0, 0, 0, 0, 0)"
%
%   * A single-port antenna moved along the x-axis and rotated 90 degrees by 
%     the theta-angle:
%     "gen_positions((1:100)', 0, 0, 0, pi/2, 0)"
%
%   * A four-port antenna, where each antenna is separated 0.2 length units 
%     (along the y-axis) and placed in a fixed position:
%     "gen_positions(0, [-0.3 -0.1 0.1 0.3], 0, 0, 0, 0)"

% By Ulf Carlberg.

% All input variables should be (maximum) 2-dimensional.
assert(all([ndims(x) ndims(y) ndims(z) ndims(alpha) ndims(theta) ...
    ndims(phi)]==2), 'All inputs must be maximum 2-dimensional.');

% The new size for all output matrices are the maximum size in each dimension of
% all the input matrices.
newSize = ones(1, 2);
for d = 1:2
    newSize(d) = max([size(x,d) size(y,d) size(z,d) size(alpha,d) ...
        size(theta,d) size(phi,d)]);
end

% Expand coordinate matrices.
p.x = xmat(x, newSize);
p.y = xmat(y, newSize);
p.z = xmat(z, newSize);

% Expand direction angles matrices.
p.alpha = xmat(alpha, newSize);
p.theta = xmat(theta, newSize);
p.phi   = xmat(phi  , newSize);
