function w = gen_waves(envType, N, M, polType, K, theta_min, theta_max, phi_min, phi_max)
%GEN_WAVES Generate multi-path environment by incident random plane waves.
%
%   function w = gen_waves(envType, N, M, polType, K, theta_min, theta_max,
%       phi_min, phi_max)
%
%   Generate N*M waves.
%   envType    - environment type:
%                '2D' means waves only from azimuth
%                '3D' means waves isotropically distributed
%                'Icosahedron' means 20 waves on each face of an icosahedron
%   N          - number of waves per realization
%   M          - number of realizations
%   polType    - polarization type 'Linear' / 'LinearVertical' / 
%                'LinearHorizontal' / 'CircularRight' / 'CircularLeft' / 
%                'Elliptical' 
%   K          - Rician K-faktor. Optional, leave undefined or empty for normal
%                Rayleigh fading.
%   theta_min  - Min theta angle (default  0 for 3D, always pi for 2D, unused
%                for Icosahedron)
%   theta_max  - Max theta angle (default pi for 3D, always pi for 2D, unused
%                for Icosahedron)
%   phi_min    - Min phi angle (default -pi for 2D and 3D, unused for
%                Icosahedron)
%   phi_max    - Max phi angle (default +pi for 2D and 3D, unused for
%                Icosahedron)

% TODO:
%function w = gen_waves(envType, N, M, polType, ampDist, polBalance, p)
%                'LaPlace' means LaPlacian distributed waves in azimuth
%   polBalance - polarization balance P_ver/P_hor (0=horizontal, 1=equal,
%                Inf=vertical)
%   p          - extra parameter for some distributions:
%                LaPlace: The scale parameter.

% By Ulf Carlberg.

if nargin<4, polType = 'Elliptical'; end
if nargin<5, K         =  []; end
if nargin<6, theta_min =   0; end
if nargin<7, theta_max =  pi; end
if nargin<8, phi_min   = -pi; end
if nargin<9, phi_max   = +pi; end

% theta in [0,pi], phi in [-pi,2*pi]
assert(theta_min>=0         && theta_min<=pi, 'theta_min out of range');
assert(theta_max>=theta_min && theta_max<=pi, 'theta_max out of range');
assert(phi_min>=-pi         && phi_min<=2*pi, 'phi_min out of range'  );
assert(phi_max>=phi_min     && phi_max<=2*pi, 'phi_max out of range'  );
assert(phi_max-phi_min<=2*pi+eps            , 'phi_max out of range'  );

w.envType = envType;
w.polType = polType;

% Define wave directions.
if strcmpi(envType, '2D')
    w.theta = (pi/2)*ones(N, M);
elseif strcmpi(envType, '3D')
    w.theta = acos( cos(theta_min) + ...
        (cos(theta_max)-cos(theta_min)) * rand(N, M) );
elseif strcmpi(envType, 'Icosahedron')
    if N~=20
        error('Number of waves per realization must be 20 for an icosahedron');
    end
    gr = (1+sqrt(5))/2;  % golden ratio
    gi = 1/gr;  % golden ratio inverse
    coord = [ -1  -1  -1 ;  -1  -1  +1 ;  -1  +1  -1 ;  -1  +1  +1 ; ...
              +1  -1  -1 ;  +1  -1  +1 ;  +1  +1  -1 ;  +1  +1  +1 ; ...
               0 -gi -gr ;   0 -gi +gr ;   0 +gi -gr ;   0 +gi +gr ; ...
             -gi -gr   0 ; -gi +gr   0 ; +gi -gr   0 ; +gi +gr   0 ; ...
             -gr   0 -gi ; -gr   0 +gi ; +gr   0 -gi ; +gr   0 +gi];
                % from Wikipedia
      [~, theta, phi] = xyz2rthetaphi(coord(:,1), coord(:,2), coord(:,3));
      w.theta = repmat(theta, 1, M);
      w.phi   = repmat(phi  , 1, M);
else
    error('Unknown environment type.');
end
if strcmpi(envType, '2D') || strcmpi(envType, '3D')
    w.phi = phi_min + (phi_max-phi_min)*rand(N, M);
elseif strcmpi(envType, 'icosahedron')
    %pass
else
    error('Unknown environment type.');
end

% Define amplitude and phase of incoming waves.
amp = complex(randn(N, M), randn(N, M));

if strcmpi(envType, 'Icosahedron')
    % Special normalization.
    pow = sum(abs(amp).^2, 2);
    amp = amp .* repmat(sqrt(M./(N*pow)), 1, M);
elseif ~isempty(K)
    % Rician fading.
    if N<2
        error(['There must be at least two waves per realization for ' ...
            'Rician fading.']);
    end
    pow_los  =     abs(amp(1    ,:)).^2;
    pow_nlos = sum(abs(amp(2:end,:)).^2, 1);
    amp(1    ,:) = amp(1    ,:) ./ sqrt((1+1/K)*pow_los);
    amp(2:end,:) = amp(2:end,:) ./ repmat(sqrt((1+K)*pow_nlos), N-1, 1);
else
    % Normalize so that the total power for each realization is one.
    pow = sum(abs(amp).^2, 1);
    amp = amp ./ repmat(sqrt(pow), N, 1);
end
    
% Define polarizations.
if strcmpi(polType, 'Linear')
    if strcmpi(envType, 'Icosahedron')
        psi = repmat(2*pi*rand(N,1), 1, M);
    else
        psi = 2*pi*rand(N, M);
    end
    w.E_theta = cos(psi) .* amp;
    w.E_phi   = sin(psi) .* amp;
elseif strcmpi(polType, 'LinearVertical')    
    w.E_theta = amp;
    w.E_phi   = complex(zeros(size(amp)));
elseif strcmpi(polType, 'LinearHorizontal')    
    w.E_theta = complex(zeros(size(amp)));
    w.E_phi   = amp;
elseif strcmpi(polType, 'CircularRight')
    w.E_theta =     amp/sqrt(2);
    w.E_phi   = -1j*amp/sqrt(2);
elseif strcmpi(polType, 'CircularLeft')
    w.E_theta =     amp/sqrt(2);
    w.E_phi   =  1j*amp/sqrt(2);
elseif strcmpi(polType, 'Elliptical')
    Eth = complex(randn(N, M), randn(N, M));
    Eph = complex(randn(N, M), randn(N, M));
    if ~isempty(K)
        % Rician fading.
        pow_los  =     hypot(Eth(1    ,:),Eph(1    ,:)).^2;
        pow_nlos = sum(hypot(Eth(2:end,:),Eph(2:end,:)).^2, 1);
        norm_factor = 1 ./ [sqrt((1+1/K)*pow_los) ; ...
            repmat(sqrt((1+K)*pow_nlos), N-1, 1)];
    else
        pow = sum(hypot(Eth,Eph).^2, 1);
        norm_factor = 1 ./ repmat(sqrt(pow), N, 1);
    end
    w.E_theta = norm_factor .* Eth;
    w.E_phi   = norm_factor .* Eph;
else
    error('Unknown polarization type.');
end

% Include dimensions in output structure.
w.nWavesPerRealization = N;
w.nRealizations = M;
