function y = get_cdf_level(x, unit, level)
%GET_CDF_LEVEL Get a certain CDF level along dimension 1.
%
%   function y = get_cdf_level(x, unit, level)
%
%   Get CDF level "level" of amplitude data "x" along dimension 1. Linear 
%   interpolation. Unit can be 'P' (power), 'dB' or 'dBR' and must be specified.
%   At level "level", default level is 1%.

% By Ulf Carlberg.

if nargin<3 || isempty(level), level=0.01; end

% Lower index to CDF.
siz = size(x);
indx_f = level*siz(1);
indx_i = floor(indx_f);
dx = indx_f - indx_i;
if indx_i<100
    error('Too few samples to determine CDF level accurately.');
end
if indx_i+1>siz(1)
    error('CDF level too large.');
end

% Init output.
y = zeros([1 siz(2:end)]);

% Calculate CDF level with linear interpolation.
for ii = 1:prod(siz(2:end))
    pow = sort(abs(x(:, ii)).^2, 1, 'ascend');
    y(1, ii) = (1-dx)*pow(indx_i) + dx*pow(indx_i+1);
end

% Assume "x" is amplitude. Convert output to user-specified unit.
if strcmpi(unit, 'P')
    % do nothing
elseif strcmpi(unit, 'dB')
    y = 10*log10(y);
elseif strcmpi(unit, 'dBR')
    r = 0.5;  % dB relative Rayleigh with expectation value 0.5.
    y = 10*log10(y) - 10*log10(-r*log(1-level));
elseif strcmpi(unit, 'dBR1')
    r = 1;  % dB relative Rayleigh with expectation value 1.
    y = 10*log10(y) - 10*log10(-r*log(1-level));
else
    error('Unknown unit.');
end
