function b = isodd(x)
%ISODD True for odd numbers.
%   ISODD(X) is 1 for the elements of X that are odd and 0 for even elements.
%
%   Class support for input X:
%      float: double, single
%      integer: [u]int8, [u]int16, [u]int32, [u]int64
%
%   An error is raised if X is a float and is:
%   * too large or small to determine if it is odd or even (this includes -Inf
%     and +Inf), or
%   * contains a fraction (e.g. isodd(1.5) raises an error), or
%   * NaN (not-a-number).
%
%   See also ISEVEN.

% By Ulf Carlberg 1-JUNE-2011.

% For floats, MOD is faster than BITGET:
%
%x=round(1e6*(rand(5000)-0.5));
%tic; b=logical(mod(x,2)); toc;
%Elapsed time is 0.912988 seconds.
%tic; b=logical(bitget(abs(x),1)); toc;
%Elapsed time is 1.431599 seconds.

% For integers, BITGET is significantly faster than MOD, even when we have to
% convert signed integers to unsigned integers:
%
%x=int32(round(1e6*(rand(5000)-0.5)));
%tic; b=logical(mod(x,2)); toc;
%Elapsed time is 2.408384 seconds.
%tic; b=logical(bitget(uint32(abs(x)),1)); b(x==-2^31)=0; toc;
%Elapsed time is 0.618207 seconds.

if isa(x, 'float') && isreal(x)
    if any(floor(x(:))~=x(:))
        error('isodd:InputNotInt', 'All entries of X must be integers.');
    end
    if any(isnan(x(:)))
        error('isodd:InputNaN', 'No entries of X are allowed to be NaN.');
    end
    if isa(x, 'single')
        if any(x(:)>=2^24)  % also x=Inf
            error('isodd:InputOutOfRange', ...
                'The maximum value of X allowed is 2^24-1 for singles.');
        end
        if any(x(:)<=-2^24)  % also x=-Inf
            error('isodd:InputOutOfRange', ...
                'The minimum value of X allowed is -2^24+1 for singles.');
        end
    else  % double
        if any(x(:)>=2^53)  % also x=Inf
            error('isodd:InputOutOfRange', ...
                'The maximum value of X allowed is 2^53-1 for doubles.');
        end
        if any(x(:)<=-2^53)  % also x=-Inf
            error('isodd:InputOutOfRange', ...
                'The minimum value of X allowed is -2^53+1 for doubles.');
        end
    end
    b = logical(mod(x, 2));
elseif isa(x, 'uint8') || isa(x, 'uint16') || isa(x, 'uint32') || ...
        isa(x, 'uint64')
    b = logical(bitget(x, 1));
elseif isa(x, 'int8')
    b = logical(bitget(uint8(abs(x)), 1));
    b(x==-2^7) = 0;  % The "weird number" in two complement form.
elseif isa(x, 'int16')
    b = logical(bitget(uint16(abs(x)), 1));
    b(x==-2^15) = 0;  % The "weird number" in two complement form.
elseif isa(x, 'int32')
    b = logical(bitget(uint32(abs(x)), 1));
    b(x==-2^31) = 0;  % The "weird number" in two complement form.
elseif isa(x, 'int64')
    b = logical(bitget(uint64(abs(x)), 1));
    b(x==-2^63) = 0;  % The "weird number" in two complement form.
else
    error('isodd:UnsupportedClass', 'Unsupported class.');
end
