function res = mimo_algorithms(H, P_t_dB)
%MIMO_ALGORITHMS Calculates MIMO/diversity combinations with many algorithms
%
%   function res = mimo_algorithms(H, P_t_dB)
%
%   Return a structure "res" with MIMO/diversity combinations from the channel
%   matrix "H" using threshold "P_t_dB" (default 0 dB). Also vectors for CDF and
%   (cumulative distribution function), CCDF (complementary CDF), and PoD
%   (probability of detection) curves are generated.
%
%   These MIMO/diversity algorithms are used:
%       "P"   - Direct power between each tx antenna to each rx antenna.
%       "SVD" - Singular value decomposition.
%       "ZF"  - Zero-Forcing (if Nrx>=Ntx).
%       "MRC" - Maximum Ratio Combination.
%       "MAX" - Selection combining (maximum).
%       "MIN" - Minimum selection.

% By Ulf Carlberg.

if nargin<2, P_t_dB = 0; end

% Vector defining theoretical Rayleigh curves to generate.
rayleigh_r = [0.5 1];

% Init.
siz = [size(H) 1];
Nrx = siz(1);
Ntx = siz(2);
N = prod(siz(3:end));
Nstreams = min(Nrx, Ntx);
Nzf = Nstreams*(Nrx>=Ntx);
res.Nrx = Nrx;
res.Ntx = Ntx;
res.N = N;
res.Nstreams = Nstreams;
res.Nzf = Nzf;

% Generate vectors for plotting CDF, CCDF.
res.CDF = (1:N)' / N;
res.CCDF = 1 - res.CDF;

% Init matrix.
NP = Nrx*Ntx + Nstreams + Nzf + 3*Ntx + numel(rayleigh_r);
res.P = zeros(N, NP);
res.names = cell(1, NP);
offset = 0;

% Power "P"-matrix.
for i2 = 1:Ntx
    for i1 = 1:Nrx
        res.names{offset+i1+(i2-1)*Nrx} = sprintf('P%d%d', i1, i2);
    end
end
res.P(:, offset+(1:Nrx*Ntx)) = reshape(abs(H).^2, Nrx*Ntx, N).';  % P
offset = offset + Nrx*Ntx;

% Singular value decomposition (SVD).
for i2 = 1:Nstreams
    res.names{offset+i2} = sprintf('SVD%d', i2);
end    
for i3 = 1:N
    res.P(i3, offset+(1:Nstreams)) = svd(H(:,:,i3)).^2;
end
offset = offset + Nstreams;

% Zero forcing (ZF).
if Nzf>0
    for i2 = 1:Nzf
        res.names{offset+i2} = sprintf('ZF%d', i2);
    end
    w = warning;
    warning('off', 'MATLAB:singularMatrix');  % ok, because we invert twice
    warning('off', 'MATLAB:nearlySingularMatrix');
    for i3 = 1:N
        HH = H(:,:,i3)' * H(:,:,i3);
        invHH = eye(Nzf) / HH;
        if all(~isnan(HH))
            invHH(isnan(invHH)) = Inf;  % becomes zero on next line
        end
        res.P(i3, offset+(1:Nzf)) = abs(real(1 ./ diag(invHH)));
    end
    warning(w);  % restore warnings
    offset = offset + Nzf;
end

% Maximum ratio combination.
for i2 = 1:Ntx
    res.names{offset+i2} = sprintf('MRC%d', i2);
end
res.P(:, offset+(1:Ntx)) = reshape(sum(abs(H).^2,1), Ntx, []).';
offset = offset + Ntx;

% Selection combining maximum value (MAX).
for i2 = 1:Ntx
    res.names{offset+i2} = sprintf('MAX%d', i2);
end
res.P(:, offset+(1:Ntx)) = reshape(max(abs(H).^2,[],1), Ntx, []).';
offset = offset + Ntx;

% Selection combining minimum value (MIN).
for i2 = 1:Ntx
    res.names{offset+i2} = sprintf('MIN%d', i2);
end
res.P(:, offset+(1:Ntx)) = reshape(min(abs(H).^2,[],1), Ntx, []).';
offset = offset + Ntx;

% Theoretical Rayleigh data.
for i2 = 1:numel(rayleigh_r)
    r = rayleigh_r(i2);
    res.names{offset+i2} = sprintf('Rayleigh r=%g', r);
    res.P(:, offset+i2) = -r*log(res.CCDF);
end
offset = offset + numel(rayleigh_r);

% Code checking.
assert(offset==NP, 'Something is wrong in the code.');

% Avoid zero power, so we avoid -Inf dB.
min_dB = -300;
res.P = max(res.P, 10^(min_dB/10));

% Generate vectors for plotting CDF, CCDF, and PoD.
res.P_dB = 10*log10(res.P);
res.P_sorted_dB = sort(res.P_dB, 1);
res.P_t_dB = P_t_dB;
res.P_t = 10^(res.P_t_dB/10);
res.P_av = res.P_t ./ sort(res.P, 1);
res.P_av_dB = 10*log10(res.P_av);

end
