function h = plot_cdf_rayleigh(r, divType, divOrder)
%PLOT_CDF_RAYLEIGH Plot CDF of Rayleigh distribution.
%
%   function h = plot_cdf_rayleigh(r, divType, divOrder)
%
%   Plots the CDF (cumulative distribution function) for theoretical
%   Rayleigh distributions with expectation value "r" (default 0.5). Optinally 
%   diversity combined.

%   References:
%   [1] Wireless Communications by Andrea Goldsmith, Cambridge University Press
%       2005.
%
% By Ulf Carlberg.

if nargin<1 || isempty(r),        r        = 0.5;    end
if nargin<2 || isempty(divType),  divType  = 'none'; end
if nargin<3 || isempty(divOrder), divOrder = 1;      end

assert(divOrder>=1, 'Diversity order must be at least 1.');

% Theory.
p_dB_sort_theory = (-40:0.1:20).';
p_sort_theory = 10.^(p_dB_sort_theory/10);
pr = p_sort_theory/r;
if strcmpi(divType, 'none')
    assert(divOrder==1, 'No diversity require diversity order 1.');
    cdf_theory = 1 - exp(-pr);
elseif strcmpi(divType, 'SC')  % selection combining
    cdf_theory = (1-exp(-pr)) .^ divOrder;  % Eq. (7.8) page 209 in [1]
elseif strcmpi(divType, 'MRC')  % maximum ratio combining
    s = 1;
    for k = 2:divOrder
        s = s + pr.^(k-1) / factorial(k-1);  % Eq. (7.19) page 214 in [1]
    end
    cdf_theory = 1 - s.*exp(-pr);  % Eq. (7.19) page 214 in [1]
else
    error('Unknown diversity scheme.');
end

% Plot.
if divOrder==1
    plot_style = 'k-';
else
    plot_style = 'k--';
end
h = semilogy(p_dB_sort_theory, cdf_theory, plot_style, 'LineWidth', 1);
