function plot_farfield(a, plotFormat, shadingType, xyzTrans, iFreq, withRingsAndAxis)
%PLOT_FARFIELD Plot far-field ("radiation pattern") of antenna.
%
%   function plot_farfield(a, plotFormat, shadingType, xyzTrans, iFreq, withRingsAndAxis)
%
%   Plot far-field function of antennas as defined in antenna structure "a".
%   "plotFormat" can be 'total' (default) for plotting both polarizations added
%   together, 'theta-phi' for plotting Gtheta and Gphi (in separate subplots)
%   or 'xPrim-yPrim' for plotting Gx' and Gy' according to Ludwig's third
%   definition. "shadingType" can be 'flat', 'interp', or 'faceted' (default).
%   See SHADING. The optional argument "xyzTrans" can be used to translate the
%   spatial position of the far-field pattern.
%
%   The plotting is always normalized and proportional to (linear) power.
%
%   See also PLOT_FARFIELD_INPOS.

% By Ulf Carlberg.

% Define optional input arguments if needed.
if nargin<2 || isempty(plotFormat      ), plotFormat       = 'total';  end
if nargin<3 || isempty(shadingType     ), shadingType      = 'interp'; end
if nargin<4 || isempty(xyzTrans        ), xyzTrans         = [0 0 0];  end
if nargin<5 || isempty(iFreq           ), iFreq            = 1;        end
if nargin<6 || isempty(withRingsAndAxis), withRingsAndAxis = 1;        end

for iAnt = 1:numel(a)

    % Define unit vector R in xyz-coordinates for far-field angles.
    [th, ph] = ndgrid(a(iAnt).theta, a(iAnt).phi);
    Eth = a(iAnt).Eth(:,:,iFreq);
    Eph = a(iAnt).Eph(:,:,iFreq);
    [x, y, z] = unit_vector_rhat(th, ph);

    % Normalize and get the linear power for the chosen plot format.
    if strcmpi(plotFormat, 'theta-phi')
        norm_factor = 1 ./ max([abs(Eth(:)) ; abs(Eph(:))]);
        W1 = (norm_factor * abs(Eth)).^2;
        W2 = (norm_factor * abs(Eph)).^2;
    elseif strcmpi(plotFormat, 'xPrim-yPrim')
        norm_factor = 1 ./ max([abs(Eth(:)) ; abs(Eph(:))]);
        W1 = (norm_factor * abs( Eth.*cos(ph) - Eph.*sin(ph) )).^2;
        W2 = (norm_factor * abs( Eth.*sin(ph) + Eph.*cos(ph) )).^2;
    elseif strcmpi(plotFormat, 'total')
        W1 = abs(Eth).^2 + abs(Eph).^2;
        W1 = W1 / max(W1(:));
    else
        error('Unknown plot format.');
    end

    if strcmpi(plotFormat, 'total')
    
        % One big plot only?
        surf(xyzTrans(1)+x.*W1, xyzTrans(2)+y.*W1, xyzTrans(3)+z.*W1, W1);
        shading(gca, shadingType);
        local_plot_commands();
    else

        % Left plot.
        subplot(1, 2, 1);
        surf(xyzTrans(1)+x.*W1, xyzTrans(2)+y.*W1, xyzTrans(3)+z.*W1, W1);
        shading(gca, shadingType);
        local_plot_commands();
    
        % Right plot.
        subplot(1, 2, 2);
        surf(xyzTrans(1)+x.*W2, xyzTrans(2)+y.*W2, xyzTrans(3)+z.*W2, W2);
        shading(gca, shadingType);
        local_plot_commands();
    end
end

if withRingsAndAxis
    if ~strcmpi(plotFormat, 'total')
        error('Plotting with rings and axis not implemented for subplots.');
    end
    rr = 1.2;
    ra = 1.8;
    hold on;
    plot3([-rr ra], [0 0], [0 0], 'r-', 'LineWidth', 2);
    plot3([0 0], [-rr ra], [0 0], 'g-', 'LineWidth', 2);
    plot3([0 0], [0 0], [-rr ra], 'b-', 'LineWidth', 2);
    text(ra+0.1, 0, 0, 'x', 'Color', 'r', 'FontSize', 12);
    text(0, ra+0.1, 0, 'y', 'Color', 'g', 'FontSize', 12);
    text(0, 0, ra+0.1, 'z', 'Color', 'b', 'FontSize', 12);
    phi = linspace(-pi, pi, 37);
    a0 = zeros(size(phi));
    a1 = rr*cos(phi);
    a2 = rr*sin(phi);
    plot3(a0, a1, a2, 'r-', 'LineWidth', 2);
    plot3(a2, a0, a1, 'g-', 'LineWidth', 2);
    plot3(a1, a2, a0, 'b-', 'LineWidth', 2);
    axis((ra+0.1)*[-1 1 -1 1 -1 1]);
end

function local_plot_commands()
set(gca, 'Projection', 'perspective');
caxis([0 1]);
colormap(jet(256));
axis image;
axis off;
view(22, 22);
% hold on;
% plot3([0 1.2 ; 0 0 ; 0 0]', [0 0 ; 0 1.2 ; 0 0]', [0 0 ; 0 0 ; 0 1.2]', 'k-');
% text(1.3, 0, 0, 'x');
% text(0, 1.3, 0, 'y');
% text(0, 0, 1.3, 'z');
