function plot_waves(w, iR, enumerate_waves)
%PLOT_WAVES Plot incoming plane waves as patches.
%
%   function plot_waves(w, iR)
%
%   Plot waves of one realization. "w" is the waves and "iR" is the realization
%   number (optional, default 1).

% By Ulf Carlberg.

if nargin<2 || isempty(iR),              iR = 1;              end
if nargin<3 || isempty(enumerate_waves), enumerate_waves = 0; end

% Patch position radius and patch width.
rp = 10 + angle(w.E_theta(:, iR))/(2*pi);
d = 1;

% xyz-coordinates for the center of patches:
[xp, yp, zp] = rthetaphi2xyz(rp, w.theta(:, iR), w.phi(:, iR));

% xyz-coordinates of spherical unit vectors at patches.
[uTx, uTy, uTz] = unit_vector_thetahat(w.theta(:, iR), w.phi(:, iR));
[uPx, uPy, uPz] = unit_vector_phihat(w.phi(:, iR));

% Patches spanned by vectors [xp1, yp1, zp1] and [xp2, yp2, zp2]:
xp1 = d/2 * uTx;
yp1 = d/2 * uTy;
zp1 = d/2 * uTz;
xyz2 = cross([xp, yp, zp], [xp1, yp1, zp1], 2)./repmat(rp, 1, 3);
xp2 = xyz2(:, 1);
yp2 = xyz2(:, 2);
zp2 = xyz2(:, 3);

% Plot the patches.
hold off;
patch([xp-xp1-xp2 xp+xp1-xp2 xp+xp1+xp2 xp-xp1+xp2]', ...
      [yp-yp1-yp2 yp+yp1-yp2 yp+yp1+yp2 yp-yp1+yp2]', ...
      [zp-zp1-zp2 zp+zp1-zp2 zp+zp1+zp2 zp-zp1+zp2]', 'r');
hold on;

% Calculate and plot polarization ellipses on patches.
normFactor = 0.5*d / max(abs([w.E_theta(:) ; w.E_phi(:)]));
Eth = normFactor*w.E_theta(:, iR);
Eph = normFactor*w.E_phi(:, iR);
phaseFactor = exp(1j*linspace(0, 2*pi, 17));
xo1 = bsxfun(@plus, xp, real((uTx.*Eth)*phaseFactor) + real((uPx.*Eph)*phaseFactor));
yo1 = bsxfun(@plus, yp, real((uTy.*Eth)*phaseFactor) + real((uPy.*Eph)*phaseFactor));
zo1 = bsxfun(@plus, zp, real((uTz.*Eth)*phaseFactor) + real((uPz.*Eph)*phaseFactor));
plot3(xo1.', yo1.', zo1.', 'k-', 'linewidth', 1);

% Plot coordinate axis.
plot3([0 3 ; 0 0 ; 0 0]', [0 0 ; 0 3 ; 0 0]', [0 0 ; 0 0 ; 0 3]', 'k-');
text(4, 0, 0, 'x');
text(0, 4, 0, 'y');
text(0, 0, 4, 'z');

% Show wave numbers.
if enumerate_waves
    for iw = 1:numel(xp)
        text(xp(iw), yp(iw), zp(iw), num2str(iw));
    end
end

% Define figure settings.
set(gca, 'Projection', 'perspective');
caxis([0 1]);
colormap(jet(256));
axis image;
axis off;
view(22, 22);
xlim([-9 9]); ylim([-9 9]); zlim([-9 9]);
