function Y = rand_laplace_trunc(siz, mu, sigma, a, b)
%RAND_LAPLACE_TRUNC Truncated Laplacian distributed pseudorandom numbers.
%
%   function Y = rand_laplace_trunc(siz, mu, sigma, a, b)
%
%   Generate random numbers for the truncated Laplacian distribution.
%   siz    - size of output
%   mu     - mean (default value 0)
%   sigma  - scale (default value 1)
%   a      - lower limit of truncation (default value -Inf)
%   b      - upper limit of truncation (default value Inf)
%
%   Example:
%   hist(rand_laplace_trunc([1e5 1], pi/2, 0.3, 0, pi), 50)
%
%   References:
%   http://en.wikipedia.org/wiki/Laplacian_distribution
%   http://en.wikipedia.org/wiki/Truncated_distribution
%
%   See also RAND_NORMAL_TRUNC.

% By Ulf Carlberg.

% Define default values for optional arguments.
if nargin<2 || isempty(mu   ), mu    =    0; end
if nargin<3 || isempty(sigma), sigma =    1; end
if nargin<4 || isempty(a    ), a     = -Inf; end
if nargin<5 || isempty(b    ), b     =  Inf; end
  
Fa = 0.5 * ( 1 + sign(a-mu).*(1-exp(-abs(a-mu)./sigma)) );
Fb = 0.5 * ( 1 + sign(b-mu).*(1-exp(-abs(b-mu)./sigma)) );

p = (Fb-Fa).*rand(siz) + Fa;
Y = mu - sigma .* sign(p-0.5) .* log(1-2*abs(p-0.5));
