function Y = rand_normal_trunc(siz, mu, sigma, a, b)
%RAND_NORMAL_TRUNC Truncated normal (Gaussian) distributed pseudorandom numbers.
%
%   function Y = rand_normal_trunc(siz, mu, sigma, a, b)
%
%   Generate random numbers for the truncated normal distribution.
%   siz   - size of output
%   mu    - mean (default value 0)
%   sigma - standard deviation (default value 1)
%   a     - lower limit of truncation (default value -Inf)
%   b     - upper limit of truncation (default value Inf)
%
%   Example:
%   hist(rand_normal_trunc([1e5 1], pi/2, 0.7, 0, pi), 50)
%
%   Reference:
%   http://en.wikipedia.org/wiki/Normal_distribution
%   http://en.wikipedia.org/wiki/Truncated_distribution
%
%   See also RAND_LAPLACE_TRUNC.

% By Ulf Carlberg.

% Define default values for optional arguments.
if nargin<2 || isempty(mu   ), mu    =    0; end
if nargin<3 || isempty(sigma), sigma =    1; end
if nargin<4 || isempty(a    ), a     = -Inf; end
if nargin<5 || isempty(b    ), b     =  Inf; end

Fa = 0.5*erfc(-(a-mu)./sqrt(2*sigma.^2));
Fb = 0.5*erfc(-(b-mu)./sqrt(2*sigma.^2));

Y = mu + sqrt(2) * sigma .* erfinv( 2*(Fb-Fa).*rand(siz) + 2*Fa - 1 );
