function a = read_farfield(filename)
%READ_FARFIELD Read antenna far-fields from ASCII files.
%
%   function a = read_farfield(filename)
%
%   Read far-field from an ASCII file and output an antenna structure. This
%   function can handle several different formats.

% By Ulf Carlberg.

% Use this list of first lines to detect file format.
firstlines = {...
    '// CST Farfield Source File', ...
    'Theta[deg.]  Phi[deg.]  Abs(Dir.)[dBi] Abs(Theta)[dBi]', ...
    'frequency phi theta Ephi_amp_lin Etheta_amp_lin Ephi_phase_rad'};

% Read first line of ASCII file to detect file format.
f = fopen(filename);
firstline = fgetl(f);
firstline = firstline(firstline~=' ');
fclose(f);
fileformat = 0;
for ii = 1:numel(firstlines)
    tmp = firstlines{ii};
    tmp = tmp(tmp~=' ');
    if strncmpi(tmp, firstline, numel(tmp))
        fileformat = ii;
        break;
    end
end

% Define antenna structure.
a.type = 'farfield';
a.name = ['File: ' filename];
a.actions = {sprintf('read_farfield(''%s'')', filename)};

if fileformat==1
    str_ver = '// Version:';
    str_freq = '// Frequency';
    str_freq1 = '// Radiated/Accepted/Stimulated Power , Frequency';
    str_data = '// >> Phi, Theta, Re(E_Theta), Im(E_Theta), Re(E_Phi),';
    f = fopen(filename);
    s = '';
    while ~strncmpi(s, str_ver, numel(str_ver))
        s = fgetl(f);
    end
    ver = str2double(fgetl(f));
    if ver==2.1
        while ~strncmpi(s, str_freq, numel(str_freq))
            s = fgetl(f);
        end
        freq = str2double(fgetl(f));
    elseif ver==3.0
        while ~strncmpi(s, str_freq1, numel(str_freq))
            s = fgetl(f);
        end
        dummy = str2double(fgetl(f)); %#ok<NASGU>
        dummy = str2double(fgetl(f)); %#ok<NASGU>
        dummy = str2double(fgetl(f)); %#ok<NASGU>
        freq = str2double(fgetl(f));
    end
    while ~strncmpi(s, str_data, numel(str_data))
        s = fgetl(f);
    end
    rawdata = textscan(f, '%f');
    fclose(f);
    data = reshape(rawdata{1}, 6, [])';
    phi2D = data(:,1);
    ntheta = find(phi2D~=phi2D(1),1) - 1;
    nphi = length(phi2D)/ntheta;
    phi2D = reshape(phi2D, ntheta, nphi);
    theta2D = reshape(data(:,2), ntheta, nphi);
    Eth = reshape(complex(data(:,3),data(:,4)), ntheta, nphi);
    Eph = reshape(complex(data(:,5),data(:,6)), ntheta, nphi);
elseif fileformat==2
    data = read_ascii_file(filename, 2);
    freq = -1;
    phi2D = data(:,2);
    ntheta = find(phi2D~=phi2D(1), 1, 'first') - 1;
    nphi = numel(phi2D) / ntheta;
    phi2D = reshape(phi2D, ntheta, nphi);
    if size(data,2)==8  % 8 columns => [dB] units
        Eth_mag = 10.^(data(:,4)/20);
        Eph_mag = 10.^(data(:,6)/20);
    elseif size(data,2)==9  % 9 columns => [V/m] units
        Eth_mag = data(:,4);
        Eph_mag = data(:,6);
    end
    theta2D = reshape(data(:,1), ntheta, nphi);
    Eth = reshape(Eth_mag.*exp(1j*data(:,5)*pi/180), ntheta, nphi);
    Eph = reshape(Eph_mag.*exp(1j*data(:,7)*pi/180), ntheta, nphi);
elseif fileformat==3
    data = read_ascii_file(filename, 1);
    npoints_per_freq = find(data(:,1)~=data(1,1), 1, 'first') - 1;
    freq = data(1:npoints_per_freq:end,1);
    phi2D = data(1:npoints_per_freq,2)*180/pi;
    ntheta = find(phi2D~=phi2D(1), 1, 'first') - 1;
    nphi = numel(phi2D) / ntheta;
    phi2D = reshape(phi2D, ntheta, nphi);
    theta2D = reshape(data(1:npoints_per_freq,3), ntheta, nphi)*180/pi;
    Eth = reshape(data(:,5).*exp(1j*data(:,7)), ntheta, nphi, numel(freq));
    Eph = reshape(data(:,4).*exp(1j*data(:,6)), ntheta, nphi, numel(freq));
else
    error('Unknown file format.');
end

% Make sure that phi is in the interval [0..360].
tol = 1e-3;
i1 = find(phi2D(1,:)>-tol, 1, 'first');
i2 = size(phi2D,2);
if abs(sind(phi2D(1,1))-sind(phi2D(1,end)))>tol
    i3 = 1;
else
    i3 = 2;
end
phi2D = mod([phi2D(:,i1:i2,:) phi2D(:,i3:i1,:)], 360);
phi2D(:,end,:) = 360;
Eth = [Eth(:,i1:i2,:) Eth(:,i3:i1,:)];
Eph = [Eph(:,i1:i2,:) Eph(:,i3:i1,:)];

% Save data in output structure.
%a.freq = freq;
a.f = freq;
a.theta = theta2D(:,1);
a.phi = phi2D(1,:).';
a.Eth = Eth;
a.Eph = Eph;

% Check angles and make sure they are exact.
assert(abs(a.theta(1))<tol, 'theta angle wrong');
assert(abs(a.theta(end)-180)<tol, 'theta angle wrong');
assert(abs(a.phi(1))<tol, 'phi angle wrong');
assert(abs(a.phi(end)-360)<tol, 'phi angle wrong');
a.theta(1) = 0;
a.theta(end) = 180;
a.phi(1) = 0;
a.phi(end) = 360;

% Compatibility. Covert degrees to radians.
a.theta = a.theta*pi/180;
a.phi = a.phi*pi/180;

% Integrate the far-field to get the radiated power.
%a.pow_rad = integrate_farfield(a);
a.power.radiated = integrate_farfield(a);

end
