function a2 = rotate_farfield(a, rotAlpha, rotTheta, rotPhi, nTheta, nPhi, interp_method)
%ROTATE_FARFIELD Rotation of antenna far-field structure with Euler angles.
%
%   function a2 = rotate_farfield(a, rotAlpha, rotTheta, rotPhi, nTheta, nPhi, interp_method)
%
%   Rotate antenna structure "a" and output a new antenna structure. The
%   rotations are done around the z-axis with angle "rotAlpha", then around the
%   y-axis with rotation angle "rotTheta", and finally around the z-axis
%   (again) with rotation angle "rotPhi".
%
%   The optional arguments "nTheta" and "nPhi" can be used to resample the
%   far-field function.

% By Ulf Carlberg.

% First just copy the antenna structure.
a2 = a;

% Leave tag on structure.
a2.info.rotated = 'Far-field rotated with virmlab.';

% Resample if "nTheta" and "nPhi" is defined as inputs.
if nargin>4
    theta = linspace(min(a.theta), max(a.theta), nTheta)';
    phi   = linspace(min(a.phi  ), max(a.phi  ), nPhi  )';
    a2.theta = theta;
    a2.phi = phi;
else
    theta = a2.theta;
    phi = a2.phi;
end

if nargin<7, interp_method = []; end

% Avoid being exactly on north/south pole to get well-defined phi-values.
pole_eps = 1e-8;
indx = abs(sin(theta)) < pole_eps;
theta(indx) = pi*round(theta(indx)/pi) + sign(1-theta(indx))*pole_eps;

% Number of frequencies/antennas.
nFreq = size(a.Eth, 3);

% Init output sizes.
a2.Eth = zeros(size(theta,1), size(phi,1), nFreq);
a2.Eph = zeros(size(theta,1), size(phi,1), nFreq);

% Define 2D grid of theta and phi.
[theta2D, phi2D] = ndgrid(theta, phi);

% Calculate the spherical unit vectors for the far-field of the input antenna
% structure.
[uRx, uRy, uRz] = unit_vector_rhat(theta2D, phi2D);
[uTx, uTy, uTz] = unit_vector_thetahat(theta2D, phi2D);
[uPx, uPy, ~  ] = unit_vector_phihat(phi2D);

% Rotate the unit vectors R backwards to get the new "theta2Drot" and "phi2Drot"
% angles of output antenna far-field. The reason we rotate R backwards rather
% than forwards is that the function "interp2_safe" requires monotonically
% increasing data.
[uRx2, uRy2, uRz2] = zyzrot(uRx, uRy, uRz, -rotPhi, -rotTheta, -rotAlpha);
[~, theta2Drot, phi2Drot] = xyz2rthetaphi(uRx2, uRy2, uRz2);

% If "phi" is in the interval [0, 2*pi] (instead of [-pi, pi]) we must also
% change "phi2Drot" to this interval to make the interpolation work.
if any(phi>pi+0.001)
    phi2Drot(phi2Drot<0) = phi2Drot(phi2Drot<0) + 2*pi;
end

% Calculate unit vectors in new directions.
[uTx2, uTy2, uTz2] = unit_vector_thetahat(theta2Drot, phi2Drot);
[uPx2, uPy2, uPz2] = unit_vector_phihat(phi2Drot);

% Rotate the new unit vectors forwards.
[uTx2, uTy2, uTz2] = zyzrot(uTx2, uTy2, uTz2, rotAlpha, rotTheta, rotPhi);
[uPx2, uPy2, uPz2] = zyzrot(uPx2, uPy2, uPz2, rotAlpha, rotTheta, rotPhi);

for iFreq = 1:nFreq

    % Calculate the field in new directions.
    Eth = interp2_safe(a.phi, a.theta, a.Eth(:,:,iFreq), ...
        phi2Drot, theta2Drot, interp_method);
    Eph = interp2_safe(a.phi, a.theta, a.Eph(:,:,iFreq), ...
        phi2Drot, theta2Drot, interp_method);

    % Project the far-field defined with the rotated unit vectors onto the
    % proper unit vectors.
    EthRot = Eth .* (uTx.*uTx2 + uTy.*uTy2 + uTz.*uTz2) + ...
             Eph .* (uTx.*uPx2 + uTy.*uPy2 + uTz.*uPz2);
    EphRot = Eth .* (uPx.*uTx2 + uPy.*uTy2) + ...
             Eph .* (uPx.*uPx2 + uPy.*uPy2);
    a2.Eth(:,:,iFreq) = EthRot;
    a2.Eph(:,:,iFreq) = EphRot;
end
