function save_ply(a, filename, ifreq)
%SAVE_PLY Save far-field radiation pattern in CAD-file format PLY.
%
%   function save_ply(a, filename, ifreq)
%
%   Save far-field radiation pattern in CAD-file ASCII format PLY.
%   PLY formats can be imported and plotted in e.g. ParaView.

% By Ulf Carlberg.

if nargin<3, ifreq = 1; end

% Normalized radiation pattern in 2D xyz-coordinates.
W = hypot(a.Eth(:,:,ifreq), a.Eph(:,:,ifreq)).^2;
W = W / max(W(:));
[x, y, z] = rthetaphi2xyz(W, repmat(a.theta,1,size(a.phi,1)), ...
    repmat(a.phi',size(a.theta,1),1));

% Define color map for radiation pattern. Assume W is normalized.
cmap = jet(4*255);
colors = 255*cmap(round(W*(size(cmap,1)-1))+1, :);

% Define number of vertex, number of faces, and an index matrix.
nvertex = numel(x);
nface = (size(x,1)-1) * (size(x,2)-1);
imat = reshape(0:nvertex-1, size(x));

% Create new PLY-file, write header.
f = fopen(filename, 'w');
fprintf(f, 'ply\nformat ascii 1.0\ncomment %s\n', 'Ulf Carlberg');
fprintf(f, 'element vertex %d\n', nvertex);
fprintf(f, 'property float x\nproperty float y\nproperty float z\n');
fprintf(f, 'property uchar red\nproperty uchar green\nproperty uchar blue\n');
fprintf(f, 'element face %d\n', nface);
fprintf(f, 'property list uchar int vertex_indices\nend_header\n');

% Write vertex list.
for i1 = 1:nvertex
    fprintf(f, '%.4f %.4f %.4f %.0f %.0f %.0f\n', x(i1), y(i1), z(i1), ...
        colors(i1,1), colors(i1,2), colors(i1,3));
end

% Write face list.
for i1 = 1:size(x,1)-1
    for i2 = 1:size(x,2)-1
        fprintf(f, '4 %d %d %d %d\n', imat(i1,i2), imat(i1,i2+1), ...
            imat(i1+1,i2+1), imat(i1+1,i2));
    end
end

% Close file.
fclose(f);
