function save_vtk(a, filename, ifreq)
%SAVE_VTK Save far-field radiation pattern in CAD-file format VTK.
%
%   function save_vtk(a, filename, ifreq)
%
%   Save far-field radiation pattern in CAD-file ASCII format VTK.
%   VTK formats can be imported and plotted in e.g. ParaView.

% By Ulf Carlberg.

if nargin<3, ifreq = 1; end

% Normalized radiation pattern in 2D xyz-coordinates.
W = hypot(a.Eth(:,:,ifreq), a.Eph(:,:,ifreq)).^2;
W = W / max(W(:));
[x, y, z] = rthetaphi2xyz(W, repmat(a.theta,1,size(a.phi,1)), ...
    repmat(a.phi',size(a.theta,1),1));

% Hack to scale from zero exactly.
[mm, ii] = min(W(:));
if mm<1e-2
    W(ii) = 0;
end

% Define number of vertex, number of faces, and an index matrix.
nvertex = numel(x);
nface = (size(x,1)-1) * (size(x,2)-1);
imat = reshape(0:nvertex-1, size(x));

% Create new VTK-file, write header.
f = fopen(filename, 'w');
fprintf(f, '# vtk DataFile Version 2.0\n%s\n', 'Ulf Carlberg');
fprintf(f, 'ASCII\n');
fprintf(f, 'DATASET POLYDATA\n');

% Write vertex list.
fprintf(f, 'POINTS %d float\n', nvertex);
for i1 = 1:nvertex
    fprintf(f, '%.4f %.4f %.4f\n', x(i1), y(i1), z(i1));
end

% Write face list.
fprintf(f, 'POLYGONS %d %d\n', nface, 5*nface);
for i1 = 1:size(x,1)-1
    for i2 = 1:size(x,2)-1
        fprintf(f, '4 %d %d %d %d\n', imat(i1,i2), imat(i1,i2+1), ...
            imat(i1+1,i2+1), imat(i1+1,i2));
    end
end
fprintf(f, '\n');

% Write scalar data.
fprintf(f, 'POINT_DATA %d\n', nvertex);
fprintf(f, 'SCALARS power_linear float\n');
fprintf(f, 'LOOKUP_TABLE default\n');
for i1 = 1:nvertex
    fprintf(f, '%.8f\n', W(i1));
end

% Close file.
fclose(f);
