function [theta, phi] = sph_regsamp(n)
%SPH_REGSAMP Regular sampling of the unit sphere.
%
%   function [theta, phi] = sph_regsamp(n)
%
%   Regular sampling of the unit sphere with "n" number of samples. The function
%   defines a spiral from the north pole to the south pole with consant arc
%   lengths between samples. Samples are never exactly on the poles. Angles are
%   given in degrees.

% By Ulf Carlberg.

% "n" should be integer, but accept some strange inputs (just like Matlab's
% "magic" function).
n = floor(real(double(n(1))));
if n<1, theta = []; phi = []; return; end

% Sample theta so that each point have a constant delta area. (Start and end
% with half area.)
s = linspace(0, 2, 2*n+1)';
theta = acosd(1-s(2:2:2*n));

% Sample phi so that the distance between points in the spiral is constant. Use
% the inverse of the Haversine formula.
dl = sqrt(4*pi/n);
th1 = theta(1:n-1);
th2 = theta(2:n);
tmp = ( sin(dl/2)^2 - sind((th2-th1)/2).^2 ) ./ ( sind(th1) .* sind(th2) );
dphi = 2 * asind(sqrt(tmp));

% Define phi angles in interval -180 to 180 starting with -180.
phi = mod([0 ; cumsum(dphi)], 360) - 180;

end
